package com.care.demo.service;

import com.care.demo.entity.RimsDictionaries;
import com.care.common.vo.PageResVO;
import com.care.common.util.PageUtil;

import java.util.List;
import java.util.Map;



/**
 * @author nhc
 * @version 1.0
 * @description 系统设置-字典管理模块-业务逻辑层接口
 * @date 2021/02/08/19:41
 */
public interface SystemDictionariesService {


    /**
     * 无条件分页查询
     * @param page 分页对象
     * @return 父子级分页字典集合
     */
    PageUtil<RimsDictionaries> getDictionaries(PageResVO page);


    /**
     * 多条件分页查询
     * @param rimsDictionaries 字典对象用来承载搜索条件
     * @param page 分页对象
     * @return 符合搜索条件的字典集合
     */
    PageUtil<RimsDictionaries> getDictionariesByPage(RimsDictionaries rimsDictionaries, PageResVO page);


    /**
     * 字段查重
     * @param rimsDictionaries 收集需要查重的字段参数
     * @return 返回0表示字段没有重复，非零表示有重复
     */
    Integer getDictionaRepetitionCount(RimsDictionaries rimsDictionaries);


    /**
     * 添加记录
     * @param rimsDictionaries 收集需要添加的字段参数
     * @return 添加成功返回 1
     */
     Integer addDictionaries(RimsDictionaries rimsDictionaries);


    /**
     * 修改记录
     * @param rimsDictionaries 收集需要添加的字段参数
     * @return 修改成功返回 1
     */
    Integer updateDictionaries(RimsDictionaries rimsDictionaries);


    /**
     * 删除记录
     * @param id 删除记录的id（有可能是父级id）
     * @return 删除成功返回 1
     */
    Integer deleteDictionariesById(Long id);


    /**
     * 启用停用记录
     * @param rimsDictionaries 字典对象里面包含要修改数据的id和启停值
     * @return 修改成功返回 1
     */
    Integer updateDictionariesEnableById(RimsDictionaries rimsDictionaries);
    /**
     * 根据编码表dicCode,查询子节点列表
     *
     * @param dicCode
     * @return
     */
    public List<RimsDictionaries> listDictionariesByDicCode(String dicCode);
    /**
     * 根据编码表父节点dicCode,节点dicCode查询节点名称
     * @param pDicCode
     * @param dicCode
     * @return
     */
    public String getDictionariesNameByDicCode(String pDicCode,String dicCode);


    /**
     * 以名称为key ，code 为值返回map
     * @param pDicCode
     * @return
     */
    public Map<String,String> getReverseDicMapByDicCode(String pDicCode);


}